package com.huaxu.async;

import com.huaxu.entity.DayReportEntity;
import com.huaxu.service.MonitorDataService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @description 异步执行定时任务
 * @auto wangli
 * @data 2020-12-03 14:15
 */
//@Component
//@EnableScheduling   // 1.开启定时任务
//@EnableAsync        // 2.开启多线程
@Slf4j
public class MonitorDataReportTaskAsync {

    @Autowired
    private MonitorDataService monitorDataService;
    /**
     * 每个小时零分零秒执行一次
     */
//    @Async
//    @Scheduled(cron = "0 0 0/1 * * ? ")
    public void monitorDataReportForHour() {
        try{
            log.info("日报报任务开始执行");
            monitorDataService.getMonitorDataReportByHour();
            log.info("日报任务执行结束");
        }catch (Exception e){
            e.printStackTrace();
            log.info("日报任务执行异常，{}", e.getMessage());
        }

    }

    /**
     * 月报
     * 每天零点执行一次
     * 统计昨天每小时的数据
     */
//    @Async
//    @Scheduled(cron = "0 0 0 * * ? ")  //每天零点执行一次
    public void monitorDataReportForDay()   {
        try{
            log.info("月报报任务开始执行");
            monitorDataService.getMonitorDataByDay();
            log.info("月报任务执行结束");
        }catch (Exception e){
            e.printStackTrace();
            log.info("月报任务执行异常，{}", e.getMessage());
        }
    }

    /**
     * 年报
     * 每月执行一次
     * 统计上月每天的数据
     */
//    @Async
//    @Scheduled(cron = "0 0 0 1 * ? ")
    public void monitorDataReportForMonth() {
        try{
            log.info("年报报任务开始执行");
            monitorDataService.getMonitorDataByMonth();
            log.info("年报任务执行结束");
        }catch (Exception e){
            e.printStackTrace();
            log.info("年报任务执行异常，{}", e.getMessage());
        }
    }

}
