package com.huaxu.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huaxu.dto.DevicePressReportAttributeDto;
import com.huaxu.dto.DeviceReportDto;
import com.huaxu.dto.DeviceWaterReportAttributeDto;
import com.huaxu.dto.ReportDto;
import com.huaxu.entity.MonitorInfoEntity;
import com.huaxu.entity.SceneEntity;
import com.huaxu.model.AjaxMessage;
import com.huaxu.model.Pagination;
import com.huaxu.model.ResultStatus;
import com.huaxu.service.ReportService;
import com.sun.org.apache.bcel.internal.generic.NEWARRAY;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 标签信息页面控制器
 * @author WYY
 * @date 2020-12-03 16:50
 */
@RestController
@RequestMapping("/report")
@Api(tags = "日月年统计报表信息")
public class DayMonthYearReportController {
    @Autowired
    private ReportService reportService;

    @ApiOperation(value = "按年月日统计报表")
    @RequestMapping(value = "/getReportBySceneIds", method = RequestMethod.POST)
    @ResponseBody
    public AjaxMessage<Pagination<ReportDto>> getReportBySceneIds(
            @RequestParam Integer pageNum,
            @RequestParam Integer pageSize,
            @ApiParam(value = "场景IDS", required = true) @RequestParam(required = true) Long[] ids,
            @ApiParam(value = "查询年", required = false) @RequestParam(required = false) Integer year ,
            @ApiParam(value = "查询月", required = false) @RequestParam(required = false) Integer month ,
            @ApiParam(value = "查询日", required = false) @RequestParam(required = false) Integer day ,
            @ApiParam(value = "报表类型(0日 1月 2年)", required = true) @RequestParam(required = true) Integer reportType
    ) {
        IPage<ReportDto> iPage = new Page<>(pageNum, pageSize);
        iPage = reportService.getReportBySceneIds(iPage,ids,year,month,day,reportType);
        Pagination<ReportDto> pages = new Pagination<>(iPage);
        return new AjaxMessage<>(ResultStatus.OK,pages);
    }

    @ApiOperation(value = "按设备统计压力及流量年月日统计报表")
    @RequestMapping(value = "/getDevicePressReportByDeviceIds", method = RequestMethod.POST)
    @ResponseBody
    public AjaxMessage<Pagination<DevicePressReportAttributeDto>> getDevicePressReportByDeviceIds(
            @RequestParam Integer pageNum,
            @RequestParam Integer pageSize,
            @ApiParam(value = "设备IDS", required = true) @RequestParam(required = true) Long[] ids,
            @ApiParam(value = "查询年", required = false) @RequestParam(required = false) Integer year ,
            @ApiParam(value = "查询月", required = false) @RequestParam(required = false) Integer month ,
            @ApiParam(value = "查询日", required = false) @RequestParam(required = false) Integer day ,
            @ApiParam(value = "类型(1压力 2流量)", required = false) @RequestParam(required = false) Integer type ,
            @ApiParam(value = "报表类型(0日 1月 2年)", required = true) @RequestParam(required = true) Integer reportType
    ) {
        IPage<DevicePressReportAttributeDto> iPage = new Page<>(pageNum, pageSize);
        iPage = reportService.getDevicePressReportByDeviceIds(iPage,ids,year,month,day,reportType,type);
        Pagination<DevicePressReportAttributeDto> pages = new Pagination<>(iPage);
        for(DevicePressReportAttributeDto item : pages.getList())
        {
            item.setAvgValue((double)Math.round(item.getAvgValue()==null?0:item.getAvgValue()*1000)/1000);
            item.setMinValue((double)Math.round(item.getMinValue()==null?0:item.getMinValue()*1000)/1000);
            item.setMaxValue((double)Math.round(item.getMaxValue()==null?0:item.getMaxValue()*1000)/1000);
            item.setSumValue((double)Math.round(item.getSumValue()==null?0:item.getSumValue()*1000)/1000);
            item.setLatestValue((double)Math.round(item.getLatestValue()==null?0:item.getLatestValue()*1000)/1000);
        }

        return new AjaxMessage<>(ResultStatus.OK,pages);
    }

    @ApiOperation(value = "按设备统计水质年月日统计报表")
    @RequestMapping(value = "/getDevicWaterReportByDeviceIds", method = RequestMethod.POST)
    @ResponseBody
    public AjaxMessage<Pagination<DeviceWaterReportAttributeDto>> getDevicWaterReportByDeviceIds(
            @RequestParam Integer pageNum,
            @RequestParam Integer pageSize,
            @ApiParam(value = "设备IDS", required = true) @RequestParam(required = true) Long[] ids,
            @ApiParam(value = "查询年", required = false) @RequestParam(required = false) Integer year ,
            @ApiParam(value = "查询月", required = false) @RequestParam(required = false) Integer month ,
            @ApiParam(value = "查询日", required = false) @RequestParam(required = false) Integer day ,
            @ApiParam(value = "报表类型(0日 1月 2年)", required = true) @RequestParam(required = true) Integer reportType
    ) {
        IPage<DeviceWaterReportAttributeDto> iPage = new Page<>(pageNum, pageSize);
        iPage = reportService.getDevicWaterReportByDeviceIds(iPage,ids,year,month,day,reportType);
        Pagination<DeviceWaterReportAttributeDto> pages = new Pagination<>(iPage);
        for(DeviceWaterReportAttributeDto item : pages.getList())
        {
            item.setAvgCLValue((double)Math.round(item.getAvgCLValue()==null?0:item.getAvgCLValue()*1000)/1000);
            item.setAvgPHValue((double)Math.round(item.getAvgPHValue()==null?0:item.getAvgPHValue()*1000)/1000);
            item.setAvgQuValue((double)Math.round(item.getAvgQuValue()==null?0:item.getAvgQuValue()*1000)/1000);

            item.setMinCLValue((double)Math.round(item.getMinCLValue()==null?0:item.getMinCLValue()*1000)/1000);
            item.setMinPHValue((double)Math.round(item.getMinPHValue()==null?0:item.getMinPHValue()*1000)/1000);
            item.setMinQuValue((double)Math.round(item.getMinQuValue()==null?0:item.getMinQuValue()*1000)/1000);

            item.setMaxCLValue((double)Math.round(item.getMaxCLValue()==null?0:item.getMaxCLValue()*1000)/1000);
            item.setMaxPHValue((double)Math.round(item.getMaxPHValue()==null?0:item.getMaxPHValue()*1000)/1000);
            item.setMaxQuValue((double)Math.round(item.getMaxQuValue()==null?0:item.getMaxQuValue()*1000)/1000);

            item.setSumCLValue((double)Math.round(item.getSumCLValue()==null?0:item.getSumCLValue()*1000)/1000);
            item.setSumPHValue((double)Math.round(item.getSumPHValue()==null?0:item.getSumPHValue()*1000)/1000);
            item.setSumQuValue((double)Math.round(item.getSumQuValue()==null?0:item.getSumQuValue()*1000)/1000);

            item.setLatestCLValue((double)Math.round(item.getLatestCLValue()==null?0:item.getLatestCLValue()*1000)/1000);
            item.setLatestPHValue((double)Math.round(item.getLatestPHValue()==null?0:item.getLatestPHValue()*1000)/1000);
            item.setLatestQuValue((double)Math.round(item.getLatestQuValue()==null?0:item.getLatestQuValue()*1000)/1000);
        }
        return new AjaxMessage<>(ResultStatus.OK,pages);
    }
}
