package com.huaxu.service.impl;

import com.huaxu.dao.MonitorDataReportMapper;
import com.huaxu.dto.MonitorDataChartReportAttributeDto;
import com.huaxu.dto.MonitorDataChartReportDeviceDto;
import com.huaxu.dto.MonitorDataChartReportValueDto;
import com.huaxu.dto.generalView.*;
import com.huaxu.entity.DeviceEntity;
import com.huaxu.entity.MonitorDataEntity;
import com.huaxu.entity.MonitorDataValueEntity;
import com.huaxu.model.LoginUser;
import com.huaxu.service.MonitorDataReportService;
import com.huaxu.service.MonitorDataService;
import com.huaxu.util.ByteArrayUtils;
import com.huaxu.util.RedisUtil;
import com.huaxu.util.UserUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.sound.midi.Receiver;
import java.math.BigDecimal;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.Period;
import java.time.format.DateTimeFormatter;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @description
 * @auto wangli
 * @data 2020/12/7 20:23
 */
@Service
public class MonitorDataReportServiceImpl implements MonitorDataReportService {

    @Resource
    private MonitorDataReportMapper monitorDataReportMapper;

    @Autowired
    private RedisUtil redisUtil;

    @Override
    public List<MonitorDataChartReportDeviceDto> monitorDataChartReportByDay(Long sceneId, Integer year, Integer month , Integer day) {
        LoginUser loginUser = UserUtil.getCurrentUser();
        List<MonitorDataChartReportDeviceDto> monitorDataChartReportDeviceDtos =  monitorDataReportMapper.MonitorDataChartReportByDay(year,month,day,sceneId,
                loginUser.getTenantId(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        FillLoseData(monitorDataChartReportDeviceDtos,3,year,month,day);
        return monitorDataChartReportDeviceDtos;
    }

    @Override
    public List<MonitorDataChartReportDeviceDto> monitorDataChartReportByMonth(Long sceneId, Integer year, Integer month ) {
        LoginUser loginUser = UserUtil.getCurrentUser();
        List<MonitorDataChartReportDeviceDto> monitorDataChartReportDeviceDtos = monitorDataReportMapper.MonitorDataChartReportByMonth(year,month,sceneId,
                loginUser.getTenantId(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        FillLoseData(monitorDataChartReportDeviceDtos,2,year,month,1);
        return monitorDataChartReportDeviceDtos;
    }

    @Override
    public List<MonitorDataChartReportDeviceDto> monitorDataChartReportByYear(Long sceneId, Integer year) {
        LoginUser loginUser = UserUtil.getCurrentUser();
        List<MonitorDataChartReportDeviceDto> monitorDataChartReportDeviceDtos = monitorDataReportMapper.MonitorDataChartReportByYear(year,sceneId,
                loginUser.getTenantId(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        FillLoseData(monitorDataChartReportDeviceDtos,1,year,1,1);
        return monitorDataChartReportDeviceDtos;
    }

    @Override
    public List<MonitorDataChartReportDeviceDto> deviceAlarmReport(Integer type, List<Long>  sceneIds, Integer year, Integer month , Integer day) {
        LoginUser loginUser = UserUtil.getCurrentUser();
        LocalDate beginDate;
        LocalDate endDate;
        String dateType ;
        if(type != null && type ==1){
            beginDate=LocalDate.of(year,1,1);
            endDate=beginDate.plusYears(1);
            dateType="month";
        }else
        if(type != null && type ==2){
            beginDate=LocalDate.of(year,month,1);
            endDate=beginDate.plusMonths(1);
            dateType="day";
        }else
        if(type != null && type ==3){
            beginDate=LocalDate.of(year,month,day);
            endDate=beginDate.plusDays(1);
            dateType="hour";
        }else{
            return new ArrayList<>();
        }
        List<MonitorDataChartReportDeviceDto> monitorDataChartReportDeviceDtos = monitorDataReportMapper.DeviceAlarmReport(dateType,sceneIds,beginDate,endDate,
                loginUser.getTenantId(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());

        FillLoseData(monitorDataChartReportDeviceDtos,type,year,month,day);
        return monitorDataChartReportDeviceDtos;
    }

    @Override
    public List<MonitorDataChartReportDeviceDto> monitorDataQualityReport(Integer type, List<Long>  sceneIds, Integer year, Integer month , Integer day) {
        LoginUser loginUser = UserUtil.getCurrentUser();

        List<MonitorDataChartReportDeviceDto> monitorDataChartReportDeviceDtos = null;
        if(type != null && type == 3){
            monitorDataChartReportDeviceDtos = monitorDataReportMapper.MonitorDataQualityReportByDay(sceneIds,year,month,day,
                    loginUser.getTenantId(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        } else if(type != null && type == 2){
            monitorDataChartReportDeviceDtos = monitorDataReportMapper.MonitorDataQualityReportByMonth(sceneIds,year,month,
                    loginUser.getTenantId(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        } else if(type != null && type == 1){
            monitorDataChartReportDeviceDtos = monitorDataReportMapper.MonitorDataQualityReportByYear(sceneIds,year,
                    loginUser.getTenantId(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        }
        if(monitorDataChartReportDeviceDtos != null){
            FillLoseData(monitorDataChartReportDeviceDtos,type,year,month,day);
        }

        return monitorDataChartReportDeviceDtos;
    }

    @Override
    public List<MonitorDataChartReportDeviceDto> monitorDataEnergyReport(Integer type, List<Long> sceneIds, Integer year, Integer month , Integer day, Integer sceneType) {
        LoginUser loginUser = UserUtil.getCurrentUser();
        List<MonitorDataChartReportDeviceDto> monitorDataChartReportDeviceDtos = null;
        if(type != null && type == 3){
            monitorDataChartReportDeviceDtos = monitorDataReportMapper.MonitorDataEnergyReportByDay(sceneType,sceneIds,year,month,day,
                   loginUser.getTenantId(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
       } else if(type != null && type == 2){
            monitorDataChartReportDeviceDtos = monitorDataReportMapper.MonitorDataEnergyReportByMonth(sceneType,sceneIds,year,month,
                   loginUser.getTenantId(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
       } else if(type != null && type == 1){
           monitorDataChartReportDeviceDtos = monitorDataReportMapper.MonitorDataEnergyReportByYear(sceneType,sceneIds,year,
                   loginUser.getTenantId(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
       }
        if(monitorDataChartReportDeviceDtos != null){
            FillLoseData(monitorDataChartReportDeviceDtos,type,year,month,day);
        }
        return monitorDataChartReportDeviceDtos;
    }
    @Override
    public List<DeviceEntity> getDeviceBySceneTypeName(String sceneTypeName){
        LoginUser loginUser = UserUtil.getCurrentUser();
        return monitorDataReportMapper.getDeviceBySceneTypeName(sceneTypeName,loginUser.getTenantId(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
    }
    @Override
    public List<MonitorDataChartReportDeviceDto> monitorDataDeviceReport(Integer type, List<Long>  deviceIds, Integer searchType, Integer year, Integer month , Integer day){

        LoginUser loginUser = UserUtil.getCurrentUser();
        List<MonitorDataChartReportDeviceDto> monitorDataChartReportDeviceDtos = null;
        if(type != null && type == 3){
            monitorDataChartReportDeviceDtos = monitorDataReportMapper.MonitorDataDeviceReportByDay(deviceIds,searchType,year,month,day,
                    loginUser.getTenantId(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        } else if(type != null && type == 2){
            monitorDataChartReportDeviceDtos = monitorDataReportMapper.MonitorDataDeviceReportByMonth(deviceIds,searchType,year,month,
                    loginUser.getTenantId(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        } else if(type != null && type == 1){
            monitorDataChartReportDeviceDtos = monitorDataReportMapper.MonitorDataDeviceReportByYear(deviceIds,searchType,year,
                    loginUser.getTenantId(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        }
        if(monitorDataChartReportDeviceDtos != null){
            FillLoseData(monitorDataChartReportDeviceDtos,type,year,month,day);
        }
        return monitorDataChartReportDeviceDtos;
    }

    /**
     * 补充数据里日期时间缺失的
     */
    private static void FillLoseData(List<MonitorDataChartReportDeviceDto> monitorDataChartReportDeviceDtos,Integer type,Integer year,Integer month ,Integer day){
        for(MonitorDataChartReportDeviceDto monitorDataChartReportDeviceDto: monitorDataChartReportDeviceDtos){
            if(monitorDataChartReportDeviceDto != null && monitorDataChartReportDeviceDto.getAttributeData() != null){
                for(MonitorDataChartReportAttributeDto monitorDataChartReportAttributeDto:monitorDataChartReportDeviceDto.getAttributeData()){
                    if(monitorDataChartReportAttributeDto != null && monitorDataChartReportAttributeDto.getMonitorDataChartReportValue() != null){
                        String unit = monitorDataChartReportAttributeDto.getUnit();
                        unit= StringUtils.isNotBlank(unit)?unit:"";
                        List<MonitorDataChartReportValueDto> monitorDataChartReportValueDtos = monitorDataChartReportAttributeDto.getMonitorDataChartReportValue();
                        Integer maxValue =0;
                        LocalDateTime localDateTime=LocalDateTime.now();
                        Integer growingBase = 1;
                        if(type != null && type ==3){
                            growingBase = 0;
                            maxValue = 24;
                            localDateTime=LocalDateTime.of(year,month,day,0,0,0);
                        }else if(type != null && type ==2){
                            localDateTime=LocalDateTime.of(year,month,1,0,0,0);
                            maxValue = localDateTime.with(TemporalAdjusters.lastDayOfMonth()).getDayOfMonth();
                        }else if(type != null && type ==1){
                            localDateTime=LocalDateTime.of(year,1,1,0,0,0);
                            maxValue=12;
                        }
                        for(int i=0;i<maxValue;i++){
                            if(i == monitorDataChartReportValueDtos.size() || !monitorDataChartReportValueDtos.get(i).getDateLabel().equals(i+growingBase)){
                                MonitorDataChartReportValueDto monitorDataChartReportValueDto = new MonitorDataChartReportValueDto();
                                monitorDataChartReportValueDto.setData(0.0);
                                LocalDateTime dateTime=localDateTime.now();
                                if(type != null && type ==3){
                                    dateTime= localDateTime.plusHours(i);
                                    monitorDataChartReportValueDto.setDateStringLabel(
                                            dateTime.getYear()+"-"+
                                                    String.format("%02d",dateTime.getMonthValue())+"-"+
                                                    String.format("%02d",dateTime.getDayOfMonth())+" "+
                                                    String.format("%02d",dateTime.getHour())+":00");
                                }
                                if(type != null && type ==2){
                                    dateTime= localDateTime.plusDays(i);
                                    monitorDataChartReportValueDto.setDateStringLabel(
                                            dateTime.getYear()+"-"+
                                                    String.format("%02d",dateTime.getMonthValue())+"-"+
                                                    String.format("%02d",dateTime.getDayOfMonth()));
                                }
                                if(type != null && type ==1){
                                    dateTime= localDateTime.plusMonths(i);
                                    monitorDataChartReportValueDto.setDateStringLabel(
                                            dateTime.getYear()+"-"+
                                                    String.format("%02d",dateTime.getMonthValue()));
                                }
                                monitorDataChartReportValueDto.setDate(dateTime);

                                monitorDataChartReportValueDto.setDateLabel(i+growingBase);
                                monitorDataChartReportValueDto.setMonitorData("");
                                monitorDataChartReportValueDtos.add(i,monitorDataChartReportValueDto);
                            }else{
                                MonitorDataChartReportValueDto monitorDataChartReportValueDto =monitorDataChartReportValueDtos.get(i);
                                monitorDataChartReportValueDto.setMonitorData(monitorDataChartReportValueDto.getData()+""+unit);
                                LocalDateTime dateTime = monitorDataChartReportValueDto.getDate();
                                if(type != null && type ==3){
                                    monitorDataChartReportValueDto.setDateStringLabel(
                                            dateTime.getYear()+"-"+
                                                    String.format("%02d",dateTime.getMonthValue())+"-"+
                                                    String.format("%02d",dateTime.getDayOfMonth())+" "+
                                                    String.format("%02d",dateTime.getHour())+":00");
                                }
                                if(type != null && type ==2){
                                    monitorDataChartReportValueDto.setDateStringLabel(
                                            dateTime.getYear()+"-"+
                                                    String.format("%02d",dateTime.getMonthValue())+"-"+
                                                    String.format("%02d",dateTime.getDayOfMonth()));
                                }
                                if(type != null && type ==1){
                                    monitorDataChartReportValueDto.setDateStringLabel(
                                            dateTime.getYear()+"-"+
                                                    String.format("%02d",dateTime.getMonthValue()));
                                }
                            }
                        }
                    }
                }
            }
        }
    }



    @Override
    public GeneralViewResults monitorDataGeneralView() {
        GeneralViewResults generalViewResults = new GeneralViewResults();
//        LoginUser loginUser = UserUtil.getCurrentUser();
//        String tenantId = loginUser.getTenantId();
//        //查询设备编码
//        List<String> deviceCodes = monitorDataReportMapper.selectDeviceCode(tenantId);
//        //根据编码获取redis中的实时数据
//        List<MonitorDataEntity> monitorDataEntities = new ArrayList<>();
//        for (String deviceCode : deviceCodes) {
//            //先取缓存里的数据
//            byte[] bytes = redisUtil.get(("sms_water_" + deviceCode).getBytes());
//            if (bytes != null && bytes.length > 0) {
//                monitorDataEntities.add((MonitorDataEntity) ByteArrayUtils.bytesToObject(bytes).get());
//            }
//        }
//        //将获取到 实时数据转换为map，方便匹配数据deviceId
//        Map<Integer, MonitorDataEntity> MonitorDataMap = monitorDataEntities.stream().collect(Collectors.toMap(MonitorDataEntity::getDeviceId, a -> a, (k1, k2) -> k1));
//
//
//        LocalDate localDate = LocalDate.now();
//        //水厂近30天供水量
//        List<DeviceWaterSupply> deviceWaterSupplies = monitorDataReportMapper.getDeviceWaterFor30Day(tenantId);
//        for (int i = 0; i < 30; i++) {
//            LocalDate newLocalDate = localDate.plusDays(-i - 1);
//            if (i <= deviceWaterSupplies.size()
//                    || !newLocalDate.isEqual(LocalDate.of(deviceWaterSupplies.get(i).getYear(),
//                    deviceWaterSupplies.get(i).getMonth(),
//                    deviceWaterSupplies.get(i).getDay()))) {
//
//                DeviceWaterSupply deviceWaterSupply = new DeviceWaterSupply();
//                deviceWaterSupply.setOrderNo(i + 1);
//                deviceWaterSupply.setAmount(BigDecimal.ZERO);
//                deviceWaterSupply.setYear(newLocalDate.getYear());
//                deviceWaterSupply.setMonth(newLocalDate.getMonthValue());
//                deviceWaterSupply.setDay(newLocalDate.getDayOfMonth());
//                deviceWaterSupply.setDate(newLocalDate);
//                deviceWaterSupplies.add(i, deviceWaterSupply);
//            } else {
//                deviceWaterSupplies.get(i).setOrderNo(i + 1);
//                deviceWaterSupplies.get(i).setDate(newLocalDate);
//            }
//        }
//        //0 30天水量
//        generalViewResults.setDeviceWaterSupplies(deviceWaterSupplies);
//
//        //供水详情数据（合计：瞬时、本天、本月、本年）
//        DeviceWaterAmount deviceWaterAmount = new DeviceWaterAmount();
//        //14瞬时流量
//        List<DeviceWaterDetail> deviceSupplyWaterDetailsForNow = monitorDataReportMapper.getDeviceWaterDetailForNow("水厂", 14, tenantId);
//        List<DeviceWaterDetail> deviceIntakeWaterDetailsForNow = monitorDataReportMapper.getDeviceWaterDetailForNow("水源", 14, tenantId);
//
//        //水厂获取实时流量并计算出当天流量
//        for (DeviceWaterDetail deviceWaterDetail : deviceSupplyWaterDetailsForNow) {
//            MonitorDataEntity monitorDataEntity = MonitorDataMap.get(deviceWaterDetail.getDeviceId());
//            if (monitorDataEntity != null && monitorDataEntity.getDataValues() != null && monitorDataEntity.getDataValues().size() > 0) {
//                for (MonitorDataValueEntity monitorDataValueEntity : monitorDataEntity.getDataValues()) {
//                    if (monitorDataValueEntity.getAttributeId().equals(deviceWaterDetail.getAttributeId())) {
//                        //水厂累计瞬时流量
//                        deviceWaterAmount.addWaterSupplyAmountForNow(new BigDecimal(monitorDataValueEntity.getDataValue()));
//                    }
//                }
//            }
//
//        }
//        //水源获取实时流量并计算出当天流量
//        for (DeviceWaterDetail deviceWaterDetail : deviceIntakeWaterDetailsForNow) {
//            MonitorDataEntity monitorDataEntity = MonitorDataMap.get(deviceWaterDetail.getDeviceId());
//            if (monitorDataEntity != null && monitorDataEntity.getDataValues() != null && monitorDataEntity.getDataValues().size() > 0) {
//                for (MonitorDataValueEntity monitorDataValueEntity : monitorDataEntity.getDataValues()) {
//                    if (monitorDataValueEntity.getAttributeId().equals(deviceWaterDetail.getAttributeId())) {
//                        //水源累计瞬时流量
//                        deviceWaterAmount.addWaterIntakeAmountForNow(new BigDecimal(monitorDataValueEntity.getDataValue()));
//                    }
//                }
//            }
//        }
//
//        //3供水 4取水
//        List<DeviceWaterDetail> deviceSupplyWaterDetailsForDay = monitorDataReportMapper.getDeviceWaterDetailForDay("水厂", 3, tenantId, localDate.getYear(), localDate.getMonthValue(), localDate.getDayOfMonth());
//        List<DeviceWaterDetail> deviceIntakeWaterDetailsForDay = monitorDataReportMapper.getDeviceWaterDetailForDay("水源", 4, tenantId, localDate.getYear(), localDate.getMonthValue(), localDate.getDayOfMonth());
//
//        //水厂获取实时流量并计算出当天流量
//        for (DeviceWaterDetail deviceWaterDetail : deviceSupplyWaterDetailsForDay) {
//            MonitorDataEntity monitorDataEntity = MonitorDataMap.get(deviceWaterDetail.getDeviceId());
//            if (monitorDataEntity != null && monitorDataEntity.getDataValues() != null && monitorDataEntity.getDataValues().size() > 0) {
//                for (MonitorDataValueEntity monitorDataValueEntity : monitorDataEntity.getDataValues()) {
//                    if (monitorDataValueEntity.getAttributeId().equals(deviceWaterDetail.getAttributeId())) {
//                        //累计当天流量
//                        deviceWaterAmount.addWaterSupplyAmountForDay(new BigDecimal(monitorDataValueEntity.getDataValue()).subtract(deviceWaterDetail.getLatestValue()).add(deviceWaterDetail.getAmount()));
//                    }
//                }
//            }
//
//        }
//        //水源获取实时流量并计算出当天流量
//        for (DeviceWaterDetail deviceWaterDetail : deviceIntakeWaterDetailsForDay) {
//            MonitorDataEntity monitorDataEntity = MonitorDataMap.get(deviceWaterDetail.getDeviceId());
//            if (monitorDataEntity != null && monitorDataEntity.getDataValues() != null && monitorDataEntity.getDataValues().size() > 0) {
//                for (MonitorDataValueEntity monitorDataValueEntity : monitorDataEntity.getDataValues()) {
//                    if (monitorDataValueEntity.getAttributeId().equals(deviceWaterDetail.getAttributeId())) {
//                        //累计当天流量
//                        deviceWaterAmount.addWaterIntakeAmountForDay(new BigDecimal(monitorDataValueEntity.getDataValue()).subtract(deviceWaterDetail.getLatestValue()).add(deviceWaterDetail.getAmount()));
//                    }
//                }
//            }
//        }
//
//        BigDecimal waterSupplyAmountForMonth = monitorDataReportMapper.getDeviceWaterDetailForMonth("水厂", 3, tenantId, localDate.getYear(), localDate.getMonthValue());
//        BigDecimal waterSupplyAmountForYear = monitorDataReportMapper.getDeviceWaterDetailForYear("水厂", 3, tenantId, localDate.getYear());
//        if (waterSupplyAmountForMonth == null) {
//            waterSupplyAmountForMonth = BigDecimal.ZERO;
//        }
//        if (waterSupplyAmountForYear == null) {
//            waterSupplyAmountForYear = BigDecimal.ZERO;
//        }
//        //水厂当月流量
//        if (deviceWaterAmount.getWaterSupplyAmountForDay() == null) {
//            deviceWaterAmount.setWaterSupplyAmountForMonth(waterSupplyAmountForMonth);
//        } else {
//            deviceWaterAmount.setWaterSupplyAmountForMonth(waterSupplyAmountForMonth.add(deviceWaterAmount.getWaterSupplyAmountForDay()));
//        }
//        //水厂当年流量
//        if (deviceWaterAmount.getWaterSupplyAmountForMonth() == null) {
//            deviceWaterAmount.setWaterSupplyAmountForYear(waterSupplyAmountForYear);
//        } else {
//            deviceWaterAmount.setWaterSupplyAmountForYear(waterSupplyAmountForYear.add(deviceWaterAmount.getWaterSupplyAmountForMonth()));
//        }
//
//
//        BigDecimal waterIntakeAmountForMonth = monitorDataReportMapper.getDeviceWaterDetailForMonth("水源", 4, tenantId, localDate.getYear(), localDate.getMonthValue());
//        BigDecimal waterIntakeAmountForYear = monitorDataReportMapper.getDeviceWaterDetailForYear("水源", 4, tenantId, localDate.getYear());
//        if (waterIntakeAmountForMonth == null) {
//            waterIntakeAmountForMonth = BigDecimal.ZERO;
//        }
//        if (waterIntakeAmountForYear == null) {
//            waterIntakeAmountForYear = BigDecimal.ZERO;
//        }
//        //水源当月流量
//        if (deviceWaterAmount.getWaterIntakeAmountForDay() == null) {
//            deviceWaterAmount.setWaterIntakeAmountForMonth(waterIntakeAmountForMonth);
//        } else {
//            deviceWaterAmount.setWaterIntakeAmountForMonth(waterIntakeAmountForMonth.add(deviceWaterAmount.getWaterIntakeAmountForDay()));
//        }
//        //水源当年流量
//        if (deviceWaterAmount.getWaterIntakeAmountForMonth() == null) {
//            deviceWaterAmount.setWaterIntakeAmountForYear(waterIntakeAmountForYear);
//        } else {
//            deviceWaterAmount.setWaterIntakeAmountForYear(waterIntakeAmountForYear.add(deviceWaterAmount.getWaterIntakeAmountForMonth()));
//        }
//        //1
//        generalViewResults.setDeviceWaterAmount(deviceWaterAmount);
//
//        //实时报警信息
//        List<SceneAlarm> sceneAlarms = monitorDataReportMapper.getAlarmInfoForScene(tenantId);
//
//        //2报警
//        generalViewResults.setSceneAlarms(sceneAlarms);
//        //管网水质（水质 -1、8PH、10浊度、11余氯）
//        List<SceneNormalRate> SceneNormalRates = new ArrayList<>();
//        //水质
//        SceneNormalRate sceneNormalRate1 = monitorDataReportMapper.getQualificationForScene(tenantId, -1);
//        //PH
//        SceneNormalRate sceneNormalRate2 = monitorDataReportMapper.getQualificationForScene(tenantId, 8);
//        //浊度
//        SceneNormalRate sceneNormalRate3 = monitorDataReportMapper.getQualificationForScene(tenantId, 10);
//        //余氯
//        SceneNormalRate sceneNormalRate4 = monitorDataReportMapper.getQualificationForScene(tenantId, 11);
//        SceneNormalRates.add(sceneNormalRate1);
//        SceneNormalRates.add(sceneNormalRate2);
//        SceneNormalRates.add(sceneNormalRate3);
//        SceneNormalRates.add(sceneNormalRate4);
//        //3水质情况
//        generalViewResults.setSceneNormalRates(SceneNormalRates);
//
//        //压力分布
//        List<DeviceWaterDetail> devicePressureWaterDetailsForNow = monitorDataReportMapper.getDeviceWaterDetailForNow("水厂", 13, tenantId);
//
//        List<SceneWaterAmount> sceneWaterPressureAmounts = new ArrayList<>();
//
//        devicePressureWaterDetailsForNow.stream()
//                .filter(item -> StringUtils.isNotBlank(item.getSceneName()))
//                .collect(Collectors.groupingBy(item -> item.getSceneName(), Collectors.toList()))
//                .forEach((key, value) -> {
//                    SceneWaterAmount sceneWaterAmount = new SceneWaterAmount();
//                    sceneWaterAmount.setSceneName(key);
//                    sceneWaterAmount.setWaterAmount(BigDecimal.ZERO);
//                    value.stream().forEach(deviceWaterDetail -> {
//                        MonitorDataEntity monitorDataEntity = MonitorDataMap.get(deviceWaterDetail.getDeviceId());
//                        if (monitorDataEntity != null && monitorDataEntity.getDataValues() != null && monitorDataEntity.getDataValues().size() > 0) {
//                            monitorDataEntity.getDataValues().stream()
//                                    .filter(monitorDataValueEntity -> monitorDataValueEntity.getAttributeId().equals(deviceWaterDetail.getAttributeId()))
//                                    .forEach(monitorDataValueEntity ->
//                                            //累计当前压力
//                                            sceneWaterAmount.setWaterAmount(sceneWaterAmount.getWaterAmount().add(new BigDecimal(monitorDataValueEntity.getDataValue())))
//                                    );
//                        }
//                    });
//                    sceneWaterPressureAmounts.add(sceneWaterAmount);
//                });
//
//        //进行压力分组
//        SceneWaterPressure sceneWaterPressure = new SceneWaterPressure();
//        SceneWaterAmount amountMax = null;
//        SceneWaterAmount amountMin = null;
//        for (SceneWaterAmount waterPressure : sceneWaterPressureAmounts) {
//            if (StringUtils.isNotBlank(waterPressure.getSceneName()) && waterPressure.getWaterAmount() != null) {
//                BigDecimal amount = waterPressure.getWaterAmount();
//                amountMax = amountMax == null || amount.compareTo(amountMax.getWaterAmount()) == 1 ? waterPressure : amountMax;
//                amountMin = amountMin == null || amount.compareTo(amountMin.getWaterAmount()) == -1 ? waterPressure : amountMin;
//
//                if (amount.compareTo(new BigDecimal(0.15)) == -1) {
//                    sceneWaterPressure.setPressureLevel1(sceneWaterPressure.getPressureLevel1() + 1);
//                } else if (amount.compareTo(new BigDecimal(0.15)) != 1 && amount.compareTo(new BigDecimal(0.30)) == -1) {
//                    sceneWaterPressure.setPressureLevel2(sceneWaterPressure.getPressureLevel2() + 1);
//                } else if (amount.compareTo(new BigDecimal(0.30)) != -1 && amount.compareTo(new BigDecimal(0.40)) == -1) {
//                    sceneWaterPressure.setPressureLevel3(sceneWaterPressure.getPressureLevel3() + 1);
//                } else if (amount.compareTo(new BigDecimal(0.40)) != -1 && amount.compareTo(new BigDecimal(0.50)) == -1) {
//                    sceneWaterPressure.setPressureLevel4(sceneWaterPressure.getPressureLevel4() + 1);
//                } else if (amount.compareTo(new BigDecimal(0.50)) != -1) {
//                    sceneWaterPressure.setPressureLevel5(sceneWaterPressure.getPressureLevel5() + 1);
//                }
//            }
//        }
//        if (amountMax != null) {
//            sceneWaterPressure.setMaxPressureSceneName(amountMax.getSceneName());
//            sceneWaterPressure.setMaxPressureValue(amountMax.getWaterAmount());
//        }
//        if (amountMax != null) {
//            sceneWaterPressure.setMinPressureSceneName(amountMin.getSceneName());
//            sceneWaterPressure.setMinPressureValue(amountMin.getWaterAmount());
//        }
//
//        //4压力分组
//        generalViewResults.setSceneWaterPressure(sceneWaterPressure);
//
//        //今日水厂制水量排行
////        deviceSupplyWaterDetailsForDay
//        List<SceneWaterAmount> sceneWaterAmounts = new ArrayList<>();
//        deviceSupplyWaterDetailsForDay.stream()
//                .filter(item -> StringUtils.isNotBlank(item.getSceneName()))
//                .collect(Collectors.groupingBy(item -> item.getSceneName(), Collectors.toList()))
//                .forEach((key, value) -> {
//                    SceneWaterAmount sceneWaterAmount = new SceneWaterAmount();
//                    sceneWaterAmount.setSceneName(key);
//                    sceneWaterAmount.setWaterAmount(BigDecimal.ZERO);
//                    value.stream().forEach(deviceWaterDetail -> {
//                        MonitorDataEntity monitorDataEntity = MonitorDataMap.get(deviceWaterDetail.getDeviceId());
//                        if (monitorDataEntity != null && monitorDataEntity.getDataValues() != null && monitorDataEntity.getDataValues().size() > 0) {
//                            monitorDataEntity.getDataValues().stream()
//                                    .filter(monitorDataValueEntity -> monitorDataValueEntity.getAttributeId().equals(deviceWaterDetail.getAttributeId()))
//                                    .forEach(monitorDataValueEntity ->
//                                            //累计当天流量
//                                            sceneWaterAmount.setWaterAmount(sceneWaterAmount.getWaterAmount().add(
//                                                    new BigDecimal(monitorDataValueEntity.getDataValue()).subtract(deviceWaterDetail.getLatestValue()).add(deviceWaterDetail.getAmount())))
//                                    );
//                        }
//                    });
//                    sceneWaterAmounts.add(sceneWaterAmount);
//                });
//        //倒序排序
//        sceneWaterAmounts.sort(Comparator.comparing(SceneWaterAmount::getWaterAmount).reversed());
//        //5水量排行
//        generalViewResults.setSceneWaterAmounts(sceneWaterAmounts);
//
//        //水厂制水、药耗、电耗对比曲线
//        List<WaterSupplyData> waterSupplyDatas = monitorDataReportMapper.getWaterSupplyData(tenantId);
//        List<WaterSupplyChart> waterSupplyCharts = new ArrayList<>();
//        waterSupplyDatas.stream()
//                .collect(Collectors.groupingBy(item -> item.getYear() + "-" + item.getMonth() + "-" + item.getDay()))
//                .forEach((key, value) -> {
//                    WaterSupplyChart waterSupplyChart = new WaterSupplyChart();
//                    waterSupplyChart.setDate(key);
//                    value.stream().forEach(waterSupplyData -> {
//                        Period period = Period.between(LocalDate.of(waterSupplyData.getYear(), waterSupplyData.getMonth(), waterSupplyData.getDay()), localDate);
//                        waterSupplyChart.setSort(period.getDays());
//
//                        if (waterSupplyData.getParmType() == 3) { //供水
//                            waterSupplyChart.setWaterData(waterSupplyData.getAmount());
//                        } else if (waterSupplyData.getParmType() == 5) {//电耗
//                            waterSupplyChart.setPowerData(waterSupplyData.getAmount());
//                        } else if (waterSupplyData.getParmType() == 6) {//药耗
//                            waterSupplyChart.setDrugData(waterSupplyData.getAmount());
//                        }
//                    });
//                    waterSupplyCharts.add(waterSupplyChart);
//                });
//        waterSupplyCharts.sort(Comparator.comparing(WaterSupplyChart::getSort));
//
//        //6曲线图
//        generalViewResults.setWaterSupplyCharts(waterSupplyCharts);
//
        return generalViewResults;
    }

    //水厂近30天供水量
    @Override
    public List<DeviceWaterSupply> deviceWaterSupplie() {
        LoginUser loginUser = UserUtil.getCurrentUser();
        String tenantId = loginUser.getTenantId();
        List<DeviceWaterSupply> deviceWaterSupplies = monitorDataReportMapper.getDeviceWaterFor30Day(tenantId,loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());

        LocalDate localDate = LocalDate.now();

        for (int i = 0; i < 30; i++) {
            LocalDate newLocalDate = localDate.plusDays(-i - 1);
            if (i == deviceWaterSupplies.size()
                    || !newLocalDate.isEqual(LocalDate.of(deviceWaterSupplies.get(i).getYear(),
                    deviceWaterSupplies.get(i).getMonth(),
                    deviceWaterSupplies.get(i).getDay()))) {

                DeviceWaterSupply deviceWaterSupply = new DeviceWaterSupply();
                deviceWaterSupply.setOrderNo(i + 1);
                deviceWaterSupply.setAmount(BigDecimal.ZERO);
                deviceWaterSupply.setYear(newLocalDate.getYear());
                deviceWaterSupply.setMonth(newLocalDate.getMonthValue());
                deviceWaterSupply.setDay(newLocalDate.getDayOfMonth());
                deviceWaterSupply.setDate(newLocalDate);
                deviceWaterSupplies.add(i, deviceWaterSupply);
            } else {
                deviceWaterSupplies.get(i).setOrderNo(i + 1);
                deviceWaterSupplies.get(i).setDate(newLocalDate);
            }
        }
        deviceWaterSupplies.sort(Comparator.comparing(DeviceWaterSupply::getOrderNo).reversed());
        return deviceWaterSupplies;
    }

    //供水详情数据（合计：瞬时、本天、本月、本年）
    @Override
    public DeviceWaterAmount deviceWaterAmount() {
        LoginUser loginUser = UserUtil.getCurrentUser();
        String tenantId = loginUser.getTenantId();
        Map<Integer, MonitorDataEntity> MonitorDataMap = getIntegerMonitorDataEntityMap(loginUser);

        LocalDate localDate = LocalDate.now();

        DeviceWaterAmount deviceWaterAmount = new DeviceWaterAmount();
        //14瞬时流量
        List<DeviceWaterDetail> deviceSupplyWaterDetailsForNow = monitorDataReportMapper.getDeviceWaterDetailForNow("水厂", 14, tenantId,loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        List<DeviceWaterDetail> deviceIntakeWaterDetailsForNow = monitorDataReportMapper.getDeviceWaterDetailForNow("水源", 14, tenantId,loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());

        //水厂获取实时流量并计算出当天流量
        for (DeviceWaterDetail deviceWaterDetail : deviceSupplyWaterDetailsForNow) {
            MonitorDataEntity monitorDataEntity = MonitorDataMap.get(deviceWaterDetail.getDeviceId());
            if (monitorDataEntity != null && monitorDataEntity.getDataValues() != null && monitorDataEntity.getDataValues().size() > 0) {
                for (MonitorDataValueEntity monitorDataValueEntity : monitorDataEntity.getDataValues()) {
                    if (monitorDataValueEntity.getAttributeId().equals(deviceWaterDetail.getAttributeId())) {
                        //水厂累计瞬时流量
                        deviceWaterAmount.addWaterSupplyAmountForNow(new BigDecimal(monitorDataValueEntity.getDataValue()));
                    }
                }
            }

        }
        //水源获取实时流量并计算出当天流量
        for (DeviceWaterDetail deviceWaterDetail : deviceIntakeWaterDetailsForNow) {
            MonitorDataEntity monitorDataEntity = MonitorDataMap.get(deviceWaterDetail.getDeviceId());
            if (monitorDataEntity != null && monitorDataEntity.getDataValues() != null && monitorDataEntity.getDataValues().size() > 0) {
                for (MonitorDataValueEntity monitorDataValueEntity : monitorDataEntity.getDataValues()) {
                    if (monitorDataValueEntity.getAttributeId().equals(deviceWaterDetail.getAttributeId())) {
                        //水源累计瞬时流量
                        deviceWaterAmount.addWaterIntakeAmountForNow(new BigDecimal(monitorDataValueEntity.getDataValue()));
                    }
                }
            }
        }

        //3供水 4取水
        List<DeviceWaterDetail> deviceSupplyWaterDetailsForDay = monitorDataReportMapper.getDeviceWaterDetailForDay("水厂", 3, tenantId, localDate.plusDays(-1).getYear(), localDate.plusDays(-1).getMonthValue(), localDate.plusDays(-1).getDayOfMonth(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        List<DeviceWaterDetail> deviceIntakeWaterDetailsForDay = monitorDataReportMapper.getDeviceWaterDetailForDay("水源", 4, tenantId, localDate.plusDays(-1).getYear(), localDate.plusDays(-1).getMonthValue(), localDate.plusDays(-1).getDayOfMonth(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());

        //水厂获取实时流量并计算出当天流量
        for (DeviceWaterDetail deviceWaterDetail : deviceSupplyWaterDetailsForDay) {
            MonitorDataEntity monitorDataEntity = MonitorDataMap.get(deviceWaterDetail.getDeviceId());
            if (monitorDataEntity != null && monitorDataEntity.getDataValues() != null && monitorDataEntity.getDataValues().size() > 0
                && deviceWaterDetail != null && deviceWaterDetail.getLatestValue() != null) {
                for (MonitorDataValueEntity monitorDataValueEntity : monitorDataEntity.getDataValues()) {
                    if (monitorDataValueEntity.getAttributeId().equals(deviceWaterDetail.getAttributeId())) {
                        //累计当天流量
                        deviceWaterAmount.addWaterSupplyAmountForDay(new BigDecimal(monitorDataValueEntity.getDataValue()).subtract(deviceWaterDetail.getLatestValue()));
                    }
                }
            }

        }
        //水源获取实时流量并计算出当天流量
        for (DeviceWaterDetail deviceWaterDetail : deviceIntakeWaterDetailsForDay) {
            MonitorDataEntity monitorDataEntity = MonitorDataMap.get(deviceWaterDetail.getDeviceId());
            if (monitorDataEntity != null && monitorDataEntity.getDataValues() != null && monitorDataEntity.getDataValues().size() > 0
                    && deviceWaterDetail != null && deviceWaterDetail.getLatestValue() != null) {
                for (MonitorDataValueEntity monitorDataValueEntity : monitorDataEntity.getDataValues()) {
                    if (monitorDataValueEntity.getAttributeId().equals(deviceWaterDetail.getAttributeId())) {
                        //累计当天流量
                        deviceWaterAmount.addWaterIntakeAmountForDay(new BigDecimal(monitorDataValueEntity.getDataValue()).subtract(deviceWaterDetail.getLatestValue()));
                    }
                }
            }
        }

        BigDecimal waterSupplyAmountForMonth = monitorDataReportMapper.getDeviceWaterDetailForMonth("水厂", 3, tenantId, localDate.getYear(), localDate.getMonthValue(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        BigDecimal waterSupplyAmountForYear = monitorDataReportMapper.getDeviceWaterDetailForYear("水厂", 3, tenantId, localDate.getYear(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        if (waterSupplyAmountForMonth == null) {
            waterSupplyAmountForMonth = BigDecimal.ZERO;
        }
        if (waterSupplyAmountForYear == null) {
            waterSupplyAmountForYear = BigDecimal.ZERO;
        }
        //水厂当月流量
        if (deviceWaterAmount.getWaterSupplyAmountForDay() == null) {
            deviceWaterAmount.setWaterSupplyAmountForMonth(waterSupplyAmountForMonth);
        } else {
            deviceWaterAmount.setWaterSupplyAmountForMonth(waterSupplyAmountForMonth.add(deviceWaterAmount.getWaterSupplyAmountForDay()));
        }
        //水厂当年流量
        if (deviceWaterAmount.getWaterSupplyAmountForMonth() == null) {
            deviceWaterAmount.setWaterSupplyAmountForYear(waterSupplyAmountForYear);
        } else {
            deviceWaterAmount.setWaterSupplyAmountForYear(waterSupplyAmountForYear.add(deviceWaterAmount.getWaterSupplyAmountForMonth()));
        }


        BigDecimal waterIntakeAmountForMonth = monitorDataReportMapper.getDeviceWaterDetailForMonth("水源", 4, tenantId, localDate.getYear(), localDate.getMonthValue(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        BigDecimal waterIntakeAmountForYear = monitorDataReportMapper.getDeviceWaterDetailForYear("水源", 4, tenantId, localDate.getYear(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        if (waterIntakeAmountForMonth == null) {
            waterIntakeAmountForMonth = BigDecimal.ZERO;
        }
        if (waterIntakeAmountForYear == null) {
            waterIntakeAmountForYear = BigDecimal.ZERO;
        }
        //水源当月流量
        if (deviceWaterAmount.getWaterIntakeAmountForDay() == null) {
            deviceWaterAmount.setWaterIntakeAmountForMonth(waterIntakeAmountForMonth);
        } else {
            deviceWaterAmount.setWaterIntakeAmountForMonth(waterIntakeAmountForMonth.add(deviceWaterAmount.getWaterIntakeAmountForDay()));
        }
        //水源当年流量
        if (deviceWaterAmount.getWaterIntakeAmountForMonth() == null) {
            deviceWaterAmount.setWaterIntakeAmountForYear(waterIntakeAmountForYear);
        } else {
            deviceWaterAmount.setWaterIntakeAmountForYear(waterIntakeAmountForYear.add(deviceWaterAmount.getWaterIntakeAmountForMonth()));
        }

        return deviceWaterAmount;
    }

    //实时报警信息
    @Override
    public List<SceneAlarm> sceneAlarms() {
        LoginUser loginUser = UserUtil.getCurrentUser();
        String tenantId = loginUser.getTenantId();
        return monitorDataReportMapper.getAlarmInfoForScene(tenantId,loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
    }

    //管网水质（水质 -1、8PH、10浊度、11余氯）
    @Override
    public List<SceneNormalRate> SceneNormalRates() {
        LoginUser loginUser = UserUtil.getCurrentUser();
        String tenantId = loginUser.getTenantId();

        List<SceneNormalRate> SceneNormalRates = new ArrayList<>();
        //水质
        SceneNormalRate sceneNormalRate1 = monitorDataReportMapper.getQualificationForScene(tenantId, -1,loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        //PH
        SceneNormalRate sceneNormalRate2 = monitorDataReportMapper.getQualificationForScene(tenantId, 8,loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        //浊度
        SceneNormalRate sceneNormalRate3 = monitorDataReportMapper.getQualificationForScene(tenantId, 10,loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        //余氯
        SceneNormalRate sceneNormalRate4 = monitorDataReportMapper.getQualificationForScene(tenantId, 11,loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());

        if(sceneNormalRate1 == null){
            sceneNormalRate1 =new SceneNormalRate();
        }
        if(sceneNormalRate2 == null){
            sceneNormalRate2 =new SceneNormalRate();
        }
        if(sceneNormalRate3 == null){
            sceneNormalRate3 =new SceneNormalRate();
        }
        if(sceneNormalRate4 == null){
            sceneNormalRate4 =new SceneNormalRate();
        }

        sceneNormalRate1.setTypeName("水质");
        sceneNormalRate2.setTypeName("PH");
        sceneNormalRate3.setTypeName("浊度");
        sceneNormalRate4.setTypeName("余氯");
        SceneNormalRates.add(sceneNormalRate1);
        SceneNormalRates.add(sceneNormalRate2);
        SceneNormalRates.add(sceneNormalRate3);
        SceneNormalRates.add(sceneNormalRate4);

        return SceneNormalRates;
    }

    //压力分布
    @Override
    public SceneWaterPressure sceneWaterPressure(String sceneType) {

        LoginUser loginUser = UserUtil.getCurrentUser();
        String tenantId = loginUser.getTenantId();
        Map<Integer, MonitorDataEntity> MonitorDataMap = getIntegerMonitorDataEntityMap(loginUser);

        List<DeviceWaterDetail> devicePressureWaterDetailsForNow = monitorDataReportMapper.getDeviceWaterDetailForNow(sceneType, 13, tenantId,loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());

        List<SceneWaterAmount> sceneWaterPressureAmounts = new ArrayList<>();

        devicePressureWaterDetailsForNow.stream()
                .filter(item -> StringUtils.isNotBlank(item.getSceneName()))
                .collect(Collectors.groupingBy(item -> item.getSceneName(), Collectors.toList()))
                .forEach((key, value) -> {
                    SceneWaterAmount sceneWaterAmount = new SceneWaterAmount();
                    sceneWaterAmount.setSceneName(key);
                    sceneWaterAmount.setWaterAmount(BigDecimal.ZERO);
                    value.stream().forEach(deviceWaterDetail -> {
                        MonitorDataEntity monitorDataEntity = MonitorDataMap.get(deviceWaterDetail.getDeviceId());
                        if (monitorDataEntity != null && monitorDataEntity.getDataValues() != null && monitorDataEntity.getDataValues().size() > 0) {
                            monitorDataEntity.getDataValues().stream()
                                    .filter(monitorDataValueEntity -> monitorDataValueEntity.getAttributeId().equals(deviceWaterDetail.getAttributeId()))
                                    .forEach(monitorDataValueEntity -> {
                                            //累计当前压力
                                            if (monitorDataValueEntity.getDataValue() != null) {
                                                sceneWaterAmount.setWaterAmount(sceneWaterAmount.getWaterAmount().add(new BigDecimal(monitorDataValueEntity.getDataValue())));
                                            }
                                        }
                                    );
                        }
                    });
                    sceneWaterPressureAmounts.add(sceneWaterAmount);
                });

        //进行压力分组
        SceneWaterPressure sceneWaterPressure = new SceneWaterPressure();
        SceneWaterAmount amountMax = null;
        SceneWaterAmount amountMin = null;
        BigDecimal amountSum = BigDecimal.ZERO;
        for (SceneWaterAmount waterPressure : sceneWaterPressureAmounts) {
            if (StringUtils.isNotBlank(waterPressure.getSceneName()) && waterPressure.getWaterAmount() != null) {
                amountSum = amountSum.add(waterPressure.getWaterAmount());
                BigDecimal amount = waterPressure.getWaterAmount();
                amountMax = amountMax == null || amount.compareTo(amountMax.getWaterAmount()) == 1 ? waterPressure : amountMax;
                amountMin = amountMin == null || amount.compareTo(amountMin.getWaterAmount()) == -1 ? waterPressure : amountMin;

                if (amount.compareTo(new BigDecimal(0.15)) == -1) {
                    sceneWaterPressure.setPressureLevel1(sceneWaterPressure.getPressureLevel1() + 1);
                } else if (amount.compareTo(new BigDecimal(0.15)) != 1 && amount.compareTo(new BigDecimal(0.30)) == -1) {
                    sceneWaterPressure.setPressureLevel2(sceneWaterPressure.getPressureLevel2() + 1);
                } else if (amount.compareTo(new BigDecimal(0.30)) != -1 && amount.compareTo(new BigDecimal(0.40)) == -1) {
                    sceneWaterPressure.setPressureLevel3(sceneWaterPressure.getPressureLevel3() + 1);
                } else if (amount.compareTo(new BigDecimal(0.40)) != -1 && amount.compareTo(new BigDecimal(0.50)) == -1) {
                    sceneWaterPressure.setPressureLevel4(sceneWaterPressure.getPressureLevel4() + 1);
                } else if (amount.compareTo(new BigDecimal(0.50)) != -1) {
                    sceneWaterPressure.setPressureLevel5(sceneWaterPressure.getPressureLevel5() + 1);
                }
            }
        }
        if(sceneWaterPressureAmounts.size()>0){
            sceneWaterPressure.setAvgPressureValue(amountSum.divide(new BigDecimal(sceneWaterPressureAmounts.size()),2,BigDecimal.ROUND_HALF_UP));
        }
        if (amountMax != null) {
            sceneWaterPressure.setMaxPressureSceneName(amountMax.getSceneName());
            sceneWaterPressure.setMaxPressureValue(amountMax.getWaterAmount());
        }
        if (amountMax != null) {
            sceneWaterPressure.setMinPressureSceneName(amountMin.getSceneName());
            sceneWaterPressure.setMinPressureValue(amountMin.getWaterAmount());
        }
        List<Integer> pressureLevels = new ArrayList<>();
        pressureLevels.add(sceneWaterPressure.getPressureLevel1());
        pressureLevels.add(sceneWaterPressure.getPressureLevel2());
        pressureLevels.add(sceneWaterPressure.getPressureLevel3());
        pressureLevels.add(sceneWaterPressure.getPressureLevel4());
        pressureLevels.add(sceneWaterPressure.getPressureLevel5());
        sceneWaterPressure.setPressureLevels(pressureLevels);
        return sceneWaterPressure;
    }

    //今日水厂制水量排行
    @Override
    public List<SceneWaterAmount> sceneWaterAmounts() {
        LoginUser loginUser = UserUtil.getCurrentUser();
        String tenantId = loginUser.getTenantId();
        //查询设备数据
        Map<Integer, MonitorDataEntity> MonitorDataMap = getIntegerMonitorDataEntityMap(loginUser);

        LocalDate localDate = LocalDate.now();
        List<DeviceWaterDetail> deviceSupplyWaterDetailsForDay = monitorDataReportMapper.getDeviceWaterDetailForDay("水厂", 3, tenantId, localDate.plusDays(-1).getYear(), localDate.plusDays(-1).getMonthValue(), localDate.plusDays(-1).getDayOfMonth(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());

        List<SceneWaterAmount> sceneWaterAmounts = new ArrayList<>();
        deviceSupplyWaterDetailsForDay.stream()
                .filter(item -> StringUtils.isNotBlank(item.getSceneName()) && item != null && item.getLatestValue() != null )
                .collect(Collectors.groupingBy(item -> item.getSceneName(), Collectors.toList()))
                .forEach((key, value) -> {
                    SceneWaterAmount sceneWaterAmount = new SceneWaterAmount();
                    sceneWaterAmount.setSceneName(key);
                    sceneWaterAmount.setWaterAmount(BigDecimal.ZERO);
                    value.stream().forEach(deviceWaterDetail -> {
                        MonitorDataEntity monitorDataEntity = MonitorDataMap.get(deviceWaterDetail.getDeviceId());
                        if (monitorDataEntity != null && monitorDataEntity.getDataValues() != null && monitorDataEntity.getDataValues().size() > 0) {
                            monitorDataEntity.getDataValues().stream()
                                    .filter(monitorDataValueEntity ->
                                            monitorDataValueEntity.getAttributeId().equals(deviceWaterDetail.getAttributeId())
                                    )
                                    .forEach(monitorDataValueEntity ->
                                            //累计当天流量
                                            sceneWaterAmount.setWaterAmount(sceneWaterAmount.getWaterAmount().add(
                                                    new BigDecimal(monitorDataValueEntity.getDataValue()).subtract(deviceWaterDetail.getLatestValue())))
                                    );
                        }
                    });
                    sceneWaterAmounts.add(sceneWaterAmount);
                });
        //倒序排序
        sceneWaterAmounts.sort(Comparator.comparing(SceneWaterAmount::getWaterAmount).reversed());
        return sceneWaterAmounts;
    }

    //水厂制水、药耗、电耗对比曲线
    @Override
    public List<WaterSupplyChart> waterSupplyCharts(String sceneTypeName) {

        LoginUser loginUser = UserUtil.getCurrentUser();
        String tenantId = loginUser.getTenantId();

        List<WaterSupplyData> waterSupplyDatas = monitorDataReportMapper.getWaterSupplyData(tenantId,sceneTypeName,loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        List<WaterSupplyChart> waterSupplyCharts = new ArrayList<>();
        waterSupplyDatas.stream()
                .collect(Collectors.groupingBy(item -> item.getYear() + "-" + item.getMonth() + "-" + item.getDay()))
                .forEach((key, value) -> {
                    WaterSupplyChart waterSupplyChart = new WaterSupplyChart();
                    waterSupplyChart.setDate(key);
                    value.stream().forEach(waterSupplyData -> {
                        Period period = Period.between(LocalDate.of(waterSupplyData.getYear(), waterSupplyData.getMonth(), waterSupplyData.getDay()), LocalDate.now());
                        waterSupplyChart.setSort(period.getDays());

                        if (waterSupplyData.getParmType() == 3) { //供水
                            waterSupplyChart.setWaterData(waterSupplyData.getAmount());
                        } else if (waterSupplyData.getParmType() == 5) {//电耗
                            waterSupplyChart.setPowerData(waterSupplyData.getAmount());
                        } else if (waterSupplyData.getParmType() == 6) {//药耗
                            waterSupplyChart.setDrugData(waterSupplyData.getAmount());
                        }
                    });
                    waterSupplyCharts.add(waterSupplyChart);
                });

        waterSupplyCharts.sort(Comparator.comparing(WaterSupplyChart::getSort));
        for(int i=0;i<7;i++){
            if(i == waterSupplyCharts.size() || !waterSupplyCharts.get(i).getSort().equals(i+1)){
                waterSupplyCharts.add(i,new WaterSupplyChart(i+1));
            }
        }
        waterSupplyCharts.sort(Comparator.comparing(WaterSupplyChart::getSort).reversed());
        return waterSupplyCharts;
    }

    //查询设备实时数据
    private Map<Integer, MonitorDataEntity> getIntegerMonitorDataEntityMap(LoginUser loginUser) {

        //查询设备编码
        List<String> deviceCodes = monitorDataReportMapper.selectDeviceCode(loginUser.getTenantId(),loginUser.getType(),loginUser.getPermissonType(),loginUser.getProgramItemList());
        deviceCodes.add("300220070692");
        //根据编码获取redis中的实时数据
        List<MonitorDataEntity> monitorDataEntities = new ArrayList<>();
        for (String deviceCode : deviceCodes) {
            //先取缓存里的数据
            byte[] bytes = redisUtil.get(("sms_water_" + deviceCode).getBytes());
            if (bytes != null && bytes.length > 0) {
                monitorDataEntities.add((MonitorDataEntity) ByteArrayUtils.bytesToObject(bytes).get());
            }
        }
        //将获取到 实时数据转换为map，方便匹配数据deviceId
        return monitorDataEntities.stream().collect(Collectors.toMap(MonitorDataEntity::getDeviceId, a -> a, (k1, k2) -> k1));
    }

}
